CON

   _clkmode       = xtal1 + pll8x
   _xinfreq       = 10_000_000

'' Constants for grayscale display.

  BASEPIN       = 12   'Base pin for video output.
  GRAYPIN       = 15   'Pin to use for DUTY-mode gray levels.
  WIDTH         = 128  'WIDTH of grayscale area in pixels. (WIDTH & 7 == 0 AND WIDTH => 128)
  HEIGHT        = 96   'HEIGHT of grayscale area in pixels. (HEIGHT => 96)

'' Pin definitions for Propeller Backpack. Can be changed for other Propeller modules.

  SCL           = 4
  SDA           = 5
  VSYNC         = 6
  MCLK          = 7
  DATA          = 0
  HSYNC         = 3

DAT

  my_pins               byte    SCL, SDA, VSYNC, MCLK, DATA, HSYNC

OBJ

  txt : "tv_wtext"              'Windowed version of tv_text.
  cam : "PropCAM_parallel4"     '4-bit-per-pixel PropCAM driver.
  ovl : "tv_gray4_overlay"      'Grayscale overlay for NTSC output.
  img : "ImageProcess4"         '4-bit-per-pixel image processor.
 
VAR

  byte sync, hist, phist[16]

PUB  start | i, maxhist

  dira[17]~~
  dira[16]~~

  txt.start(BASEPIN)
  gray_buf_addr := @gray_buf
  sync_addr := @sync
  ovl.start(BASEPIN, GRAYPIN, @gray_buf, 95, 28, 2, @sync)
  cam.start(cam#SYNCED, @my_pins, @gray_buf_addr)
  img.start(@gray_buf)
  txt.str(string(txt#DEFWIN, 1, 2, 1, 35, 12))
  txt.str(string(txt#DEFWIN, 2, 35, 1, 5, 4, txt#USEWIN, 1, txt#USEWIN, txt#ZERO))
  txt.str(string(txt#HOME, txt#SELCLR, 12, "PropCAM Histogram Demo", txt#USEWIN, 2))
  repeat
    cam.acquire(cam#GRAB)
    img.filled_box(0, 0, 127, 95, img#UNDEF, img#UNDEF, $8000, @stats)
    maxhist~
    repeat i from 0 to 15
      maxhist #>= histogram[i]
    repeat i from 0 to 15
      hist := histogram[i] * 15 / maxhist
      if (hist > phist[i])
        img.filled_box(128 + phist[i], i * 4, 128 + hist, i * 4 + 2, 15, 15, 0, 0)
      elseif (hist < phist[i])
        img.filled_box(128 + hist + 1, i * 4, 128 + phist[i], i * 4 + 2, 0, 0, 0, 0)
      phist[i] := hist

DAT

gray_buf      long      (cam#PIXELS_PER_ROW + 16) << 16 | cam#ROWS_PER_FRAME, 0[(cam#LONGS_PER_ROW + 2) * cam#ROWS_PER_FRAME]

gray_buf_addr word      0-0                     '@gray_buf gets plugged in here.
left          word      0                       'Lefmost pixel in gray_buf for capture.
top           word      0                       'Topmost oixel in gray_buf for capture.
exp_type      word      cam#AUTOBOTH            'Set exposure mode to automatically adjust gain and exposure time.
sync_addr     word      0-0                     '@sync gets plugged in here.
exp_time      word      20                      'Start with minimum exposure time.
gain          word      $0f                     'Start with medium gain.
target        word      700                     'Autoexposure target for avg_pix.
time_interval word      -30                     'Initial exporsure time = 1/30 sec.
avg_pix       word      0-0                     'Average pixel value from last capture * 100.

stats
histogram     word      0[16]
bbx0          word      0
bby0          word      0
bbx1          word      0
bby1          word      0
ctrx          word      0
ctry          word      0